﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Media;
using C1.C1Pdf;
using System.Drawing;
using System.Windows.Forms;

namespace DataGridExportPDF
{
    /// <summary>
    /// Utility class on top of C1.Silverlight.Pdf.
    /// </summary>
    public static class PdfUtils
    {
        // ***********************************************************************
        // C1PdfDocuments for RectangleF
        // ***********************************************************************


        // measure a paragraph, skip a page if it won't fit, render it into a rectangle,
        // and update the rectangle for the next paragraph.
        // 
        // optionally mark the paragraph as an outline entry and as a link target.
        //
        // this routine will not break a paragraph across pages. for that, see the Text Flow sample.
        //
        public static RectangleF RenderParagraph(this C1PdfDocument pdf, string text, Font font, RectangleF rcPage, RectangleF rc, bool outline, bool linkTarget)
        {
            // if it won't fit this page, do a page break
            rc.Height = pdf.MeasureString(text, font, (float)rc.Width).Height;
            if (rc.Bottom > rcPage.Bottom)
            {
                pdf.NewPage();
                rc.Y = rcPage.Top;
            }

            // draw the string
            pdf.DrawString(text, font, System.Drawing.Brushes.Black, rc);
            // show bounds (to check word wrapping)
            //var p = Pen.GetPen(Colors.Orange);
            //pdf.DrawRectangle(p, rc);

            // add headings to outline
            if (outline)
            {
                pdf.DrawLine(Pens.Black, (float)rc.X, (float)rc.Y, (float)rc.Right, (float)rc.Y);
                pdf.AddBookmark(text, 0, (float)rc.Y);
            }

            // add link target
            if (linkTarget)
            {
                pdf.AddTarget(text, rc);
            }

            // update rectangle for next time
            rc = Offset(rc, 0, rc.Height);
            return rc;
        }

        public static RectangleF ConvertRect(Rect rc)
        {
            return new RectangleF((float)rc.X, (float)rc.Y, (float)rc.Width, (float)rc.Height);
        }

        public static RectangleF RenderParagraph(this C1PdfDocument doc, string text, Font font, RectangleF rcPage, RectangleF rc, bool outline)
        {
            return RenderParagraph(doc, text, font, rcPage, rc, outline, false);
        }

        public static RectangleF RenderParagraph(this C1PdfDocument doc, string text, Font font, RectangleF rcPage, RectangleF rc)
        {
            return RenderParagraph(doc, text, font, rcPage, rc, false, false);
        }

        public static RectangleF PageRectangle(this C1PdfDocument pdf)
        {
            return PageRectangle(pdf, new Thickness(72));
        }

        public static RectangleF PageRectangle(this C1PdfDocument pdf, Thickness pageMargins)
        {
            RectangleF rc = pdf.PageRectangle;
            float left = (float)Math.Min(rc.Width, rc.Left + pageMargins.Left);
            float top = (float)Math.Min(rc.Height, rc.Top + pageMargins.Top);
            float width = (float)Math.Max(0, rc.Width - (pageMargins.Left + pageMargins.Right));
            float height = (float)Math.Max(0, rc.Height - (pageMargins.Top + pageMargins.Bottom));
            return new RectangleF(left, top, width, height);
        }

        public static void Save(this C1PdfDocument pdf)
        {
            var dlg = new SaveFileDialog();
            dlg.DefaultExt = ".pdf";
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                using (var stream = dlg.OpenFile())
                {
                    pdf.Save(stream);
                }
            }
        }


        // ***********************************************************************
        // Extension methods for RectangleF
        // ***********************************************************************

        public static RectangleF Inflate(this RectangleF rc, float dx, float dy)
        {
            rc.X -= dx;
            rc.Y -= dy;
            rc.Width += 2 * dx;
            rc.Height += 2 * dy;
            return rc;
        }

        public static RectangleF Offset(this RectangleF rc, float dx, float dy)
        {
            rc.X += dx;
            rc.Y += dy;
            return rc;
        }

    }
}
